/********************************************/
/*    リレー接点抵抗チェッカー              */
/********************************************/
//  AC100Vの負荷を開閉しながらリレー接点の接触抵抗を測定する
//  Arduino UNOを使用

/*****  ピン指定        *****/
//  液晶ピン指定
#define LCD_RS    3      // デジタルピン指定
#define LCD_E     2      // 4bitモードで使用
#define LCD_D4    4
#define LCD_D5    5
#define LCD_D6    6
#define LCD_D7    7

//  リレー駆動ピン指定
#define RL_A      A0    // 検査対象対象のリレー
#define RL_B      A1    // AC100V接続リレー
#define RL_C      A2    // 定電流、A/D入力切替リレー

//  A/Dコンバータ（MCP3204）ピン指定
#define AD_CLK    8     // A/D CLK
#define AD_DOUT   9     // A/D DOUT (入力)
#define AD_DIN   10     // A/D DIN
#define AD_CS    11     // A/D /CS

//  スイッチ入力 (プルアップ)
#define SW_START 12     // 開始スイッチ
#define SW_TX    13     // データ送出スイッチ

/*****  設定値      *****/
#define  MESD_MAX     501        // 測定データ最大値
                                 // 0回目＋500万回目
#define  RL_DCYC      200        // 接点データ表示サイクル
#define  RL_MCYC      10000L     // 接点データ測定記録サイクル

//  リレー切替時間設定
#define  TM_RLSEL    100        // 測定時リレー切替時間 (x1ms)
#define  TM_RLCHOP   90         // 試験リレー on/off時間  (x1ms)
#define  TM_MWAIT    60         // 測定記録確認待ち時間 (x1秒)
#define  TM_MRTRY    10         // 測定記録リトライ待ち時間 (x1秒)

/*****  ライブラリ      ****/
#include <MsTimer2.h>            // タイマー割り込み
#include <EEPROM.h>              // EEPROM
#include <LiquidCrystal.h>       // 16文字×2行液晶を4bitモードで使用
// initialize the library with the numbers of the interface pins
LiquidCrystal lcd(LCD_RS, LCD_E, LCD_D4, LCD_D5, LCD_D6, LCD_D7);

/*****  タイトル      *****/
const char  *pgm_ttl[] = {
        "Relay Checker",
        "      2012-12-22",
    //   0123456789012345
};

/****  Ω記号    *****/
#define  CG_OHM    1      // キャラジェネデータ
byte cgd_ohm[8] = {
    B01110,
    B10001,
    B10001,
    B10001,
    B10001,
    B01010,
    B11011,
    B00000,
};

/*****  グローバルデータ    *****/
word  ad_data;                    // A/Dデータ平均値
word  ad_mem;                     // A/Dデータ記録値
                                  // 記録サイクルで安定をチェック
word  mrtry_cnt;                  // 測定安定待ちリトライ回数
//  EEPROM保持データ
word  mes_cnt;                    // 測定データ数
unsigned long chop_cnt;           // リレー開閉回数

/*****  EEPROMデータアドレス  *****/
#define  EP_MESCNT    2          // 測定データ数
#define  EP_CHOPCNT   4          // リレー開閉回数
#define  EP_MESDTOP   8          // 測定データの先頭 

/*****  フラグ        *****/
boolean    f_swon;                // SW on フラグ (ちょん押し)
boolean    f_swstart;             // SW start フラグ　(長押し)
boolean    f_swtx;                // SW データ送信
boolean    f_adon;                // A/D変換開始フラグ
boolean    f_adok;                // A/D変換終了フラグ
boolean    f_ad1ms;               // 1ms A/D変換タイミング
boolean    f_pwrlon;              // 電源リレーオン指令
boolean    f_pwrloff;             // 電源リレーオフ指令
boolean    f_mesrl;               // 測定リレー制御指令

/******************************/
/*      1ms タイマー割り込み  */
/******************************/

/*****  タイマーデータ    *****/
volatile word  tm_1ms;          // 計時用ダウンカウント
volatile word  tm_1sec;         // 1秒計時ダウンカウント
volatile word  tm_swstart;      // 開始スイッチ
                                // 長押しチェックする
volatile word  tm_swtx;         // データ送信スイッチ
volatile word  tm_rl;           // リレー切替時間


/*****  割り込み実行      *****/
void int1ms(void)
{
static word t1000 = 0;          // 1秒チェックカウンタ
    if(tm_1ms)          tm_1ms--;
    if(tm_swstart)      tm_swstart--;
    if(tm_swtx)         tm_swtx--;
    if(tm_rl)           tm_rl--;
    f_ad1ms = 1;                    // A/D開始タイミング
//  1秒計時
    t1000++;
    if(t1000 >= 1000){
      t1000 = 0;
      if(tm_1sec)       tm_1sec--;
    }
}


/****************************/
/*      A/D入力             */
/****************************/

/*****  A/D入力      *****/
//      ch0の12bit値を返す
//      およそ260usで変換
word mcp3204(byte ch)
{
byte i;
word d = 0;
    cli();                             // 割り込み禁止
    digitalWrite(AD_CS,  HIGH);        // A/D CS  = H
    digitalWrite(AD_CLK, LOW);         // A/D CLK = L
    digitalWrite(AD_CS,  LOW);         // A/D CS  = L
//  変換ch指定    
    ch |= B11000;                 // スタートビットとシングルエンドビットを付加
    ch <<= 3;                          // 計5ビットを送出
    for(i = 0; i < 7; i++){            // clkパルスは7つ
      if(ch & 0x80)   digitalWrite(AD_DIN, HIGH);     // MSBから出力
      else            digitalWrite(AD_DIN, LOW);
      digitalWrite(AD_CLK, HIGH);      // A/D CLK = H
      digitalWrite(AD_CLK, LOW);       // A/D CLK = L
      ch <<= 1;                        // 左シフト
    }
    digitalWrite(AD_DIN, HIGH);        // A/D DIN = H
//  A/D変換完了データ読み取り
    for(i = 0; i < 12; i++){            // clkパルス
      d <<= 1;
      if(digitalRead(AD_DOUT) == HIGH)    d |= 1;    // on
      digitalWrite(AD_CLK, HIGH);      // A/D CLK = H
      digitalWrite(AD_CLK, LOW);       // A/D CLK = L
    }
    digitalWrite(AD_CS,  HIGH);        // A/D CS  = H
    digitalWrite(AD_CLK, HIGH);        // A/D CLK = H
    sei();                             // 割り込み許可
    return d;
}

/*****  A/D入力平均化処理      *****/
//  f_adonで変換開始
//  1msごとに64サンプルしてad_dataに平均値を残す
void adavr(void)
{
static byte m = 0;              // 処理区分
static byte cnt;                // 加算回数
static unsigned long ad_add;    // 12bit A/D 加算値
    switch(m){
      case 0:        // f_adonを待つ
        if(f_adon){
          cnt     = 64;         // 64回加算カウンタ
          ad_add  = 0;          // A/Dデータ加算値
          f_ad1ms = 0;          // A/D入力タイミング
          m++;
        }
        break;
      case 1:        // f_ad1msを待つ
        if(f_adon == 0){        // 中断
          m = 0;
          break;
        }
        if(f_ad1ms){
          f_ad1ms = 0;                            // 次A/D入力
          ad_add += (unsigned long)mcp3204(0);    // ch0 A/D
          cnt--;
          if(cnt == 0){              // 64回おわり? 
            ad_data = ad_add / 64;   // 平均値算出
            f_adon = 0;              // 開始フラグをオフして完了
            f_adok = 1;              // 完了フラグをオン
            m = 0;                   // 次開始を待つ
          }
        }
        break;
    }       
}


/****************************/
/*      スイッチチェック    */
/****************************/

/*****  スタートスイッチ    *****/
//    ちょん押しだとf_swon
//    長押しはf_swstartをオン
void swchkstart(void)
{
static byte n = 0;
boolean sw;
    if(digitalRead(SW_START) == LOW)    sw = 1;    // on
    else                                sw = 0;    // off
    switch(n){
      case 0:        // off確認
        if(sw == 0)    n++;
        break;
      case 1:        // onを待つ
        if(sw == 1){
          tm_swstart = 10;        // 10msタイマー
          n++;
        }
        break;
      case 2:        // on安定確認
        if(sw == 0){        // offなら最初から
          n = 0;
        }
        else{                   // on継続
          if(tm_swstart == 0){    // 10ms安定
            tm_swstart = 2000;    // 長押しチェック
            n++;
          }
        }
        break;
      case 3:        // 長押し判定
        if(sw == 1){              // on継続
          if(tm_swstart == 0){    // 2s経過
            f_swstart = 1;        // START確定
            n = 0;
          }
        }
        else{                     // offした
          f_swon = 1;             // ちょん押しon確定
            n = 0;
        }  
        break;
    }
} 

/*****  データ送信スイッチ    *****/
//    f_swtxをオン
void swchktx(void)
{
static byte n = 0;
boolean sw;
    if(digitalRead(SW_TX) == LOW)    sw = 1;    // on
    else                             sw = 0;    // off
    switch(n){
      case 0:        // off確認
        if(sw == 0)    n++;
        break;
      case 1:        // onを待つ
        if(sw == 1){
          tm_swtx = 10;        // 10msタイマー
          n++;
        }
        break;
      case 2:        // on安定確認
        if(sw == 0){           // offなら最初から
          n = 0;
        }
        else{                  // on継続
          if(tm_swtx == 0){    // 10ms安定
            f_swtx = 1;        // データ送信スイッチon確定
            n = 0;             // off待ちから
          }
        }
        break;
    }
} 

/****************************/
/*      リレー切替          */
/****************************/

/*****  電源接続      *****/
//      リレーを順次切り替えて電源に接続
void pwrlon(void)
{
static byte m = 0;
    switch(m){
      case 0:        // f_pwrlonを待つ
        if(f_pwrlon){
          digitalWrite(RL_A, LOW);     // 接点試験リレーオフ
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 1:        // Aリレーオフ時間待ち
        if(tm_rl == 0){
          digitalWrite(RL_C, HIGH);    // 測定切替リレーオン
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 2:        // Cリレーオン時間待ち
        if(tm_rl == 0){
          digitalWrite(RL_B, HIGH);    // 電源側リレーオン
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 3:        // Bリレーオン時間待ち
        if(tm_rl == 0){
          f_pwrlon = 0;                // フラグクリアして
          m = 0;                       // 待機状態に
        }
        break;
    }       
}

/*****  電源切断      *****/
//      リレーを順次切り替えて電源から切断
void pwrloff(void)
{
static byte m = 0;
    switch(m){
      case 0:        // f_pwrloffを待つ
        if(f_pwrloff){
          digitalWrite(RL_A, LOW);     // 接点試験リレーオフ
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 1:        // Aリレーオフ時間待ち
        if(tm_rl == 0){
          digitalWrite(RL_B, LOW);     // 電源側リレーオフ
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 2:        // Bリレーオフ時間待ち
        if(tm_rl == 0){
          digitalWrite(RL_C, LOW);     // 測定切替リレーオフ
          tm_rl = TM_RLSEL;            // タイマーセット
          m++;
        }
        break;
      case 3:        // Cリレーオン時間待ち
        if(tm_rl == 0){
          f_pwrloff = 0;               // フラグクリアして
          m = 0;                       // 待機状態に
        }
        break;
    }       
}

/*****  測定処理      *****/
//      接点試験リレーをオンしてA/D開始
void mesrl(void)
{
static byte m = 0;
    switch(m){
      case 0:        // f_mesrlを待つ
        if(f_mesrl){
          digitalWrite(RL_A, HIGH);     // 接点試験リレー on
          tm_rl = TM_RLSEL;
          f_adon = 0;                   // A/D処理を中断
          f_adok = 0;
          m++;
        }
        break;
      case 1:        // オンタイムアップ待ち
        if(tm_rl == 0){                 // リレーオン後に安定したら
          f_adon = 1;                   // A/D平均開始
          m++;
        }
        break;
      case 2:        // A/D完了待ち
        if(f_adok){
          f_adok = 0;
          digitalWrite(RL_A, LOW);     // 接点試験リレーoff
          tm_rl = TM_RLSEL;            // オフ時間
          m++;
        }
        break;
      case 3:      // オフタイムアップ待ち          
        if(tm_rl == 0){                // 完了
          f_mesrl = 0;                 // フラグクリアして
          m = 0;                       // 待機状態に
        }
        break;
    }
}

/****************************/
/*      EEPROM              */
/****************************/

/*****  EEPROM word 読み出し  *****/
word eeprd2(word a)
{
word d;
    d = word(EEPROM.read(a+1), EEPROM.read(a));    // H,L
    return d;
}

/*****  EEPROM word 書き込み  *****/
void eepwr2(word a, word d)
{
    EEPROM.write(a,   lowByte(d));     // L
    EEPROM.write(a+1, highByte(d));    // H
}

/*****  EEPROM 4バイト 読み出し  *****/
unsigned long eeprd4(word a)
{
unsigned long d = 0L;
byte i;
    for(i = 0; i < 4; i++){
      d <<= 8;
      d |=  (unsigned long)EEPROM.read(a+3);    // +3～+0
      a--;
    }
    return d;
}

/*****  EEPROM 4バイト 書き込み  *****/
void eepwr4(word a, unsigned long d)
{
byte i;  
    for(i = 0; i < 4; i++){
      EEPROM.write(a, lowByte(d));     // +0～+3
      a++;
      d >>= 8;
    }
}


/****************************/
/*      サブルーチン        */
/****************************/

/*****  測定データ表示    *****/
void dispmesdata(void)
{
char s[16];
    sprintf(s, "mem=%-3u", mes_cnt);   // 測定データ数
    lcd.setCursor(9, 0);               // 1行目
    lcd.print(s);
    if(ad_data < 4095)
        sprintf(s, "%4u", ad_data);    // A/D平均値
    else
        strcpy(s, "----");             // オーバーフローは----表示で
    lcd.setCursor(0, 1);               // 2行目
    lcd.print(s);
    lcd.write('m');
    lcd.write(CG_OHM);                 // mΩ
    sprintf(s, "%8lu", chop_cnt);      // 開閉回数
    lcd.setCursor(8, 1);               // 2行目
    lcd.print(s);
}


/****************************/
/*        実行              */
/****************************/
//    実行区分
byte exc_j = 0;           // 実行区分
#define  J_STBY    2      // スタンバイ
#define  J_SWCHK   4      // スイッチチェック
#define  J_DTX     5      // データ送信
#define  J_RUN     6      // リレー開閉
#define  J_RUN1    7      // リレー開閉#1 電源接続
#define  J_RUN2    8      // リレー開閉#2 開閉繰り返し
#define  J_MES    11      // 測定
#define  J_MEM    14      // 測定記録チェック
#define  J_MWR    16      // 測定値EEPROM書き込み

/*****  タイトル表示      *****/
void jttl(void)
{
    lcd.setCursor(0, 0);        // １行目
    lcd.print(pgm_ttl[0]);    
    lcd.setCursor(0, 1);        //  2行目
    lcd.print(pgm_ttl[1]);    
    tm_1ms = 1000;              // 1秒待ち
    exc_j++;
}

/*****  タイトル表示完了待ち  *****/
void jttl1(void)
{
    if(tm_1ms == 0){        // タイムアップ?
      exc_j++;
    }
}

/*****  スタンバイ          *****/
//      リレーオフ
void jstby(void)
{
    f_pwrloff = 1;                // 電源接続オフ
    lcd.clear();                  // 液晶消去
    lcd.setCursor(0, 0);          // １行目
    lcd.print("Stand-by ");    
    dispmesdata();                // データ表示
    exc_j++;
}

/*****  スタンバイ＃１      *****/
void jstby1(void)
{
    if(f_pwrloff == 0){           // オフ完了を待つ
      exc_j = J_SWCHK;            // スイッチチェックへ
    }
}

/*****  スイッチ入力待ち    *****/
void jswchk(void)
{
    if(f_swtx){                   // データ送信SW
      f_swtx = 0;
      exc_j = J_DTX;  
      return;
    }
    if(f_swstart){                // クリアスタート
      f_swstart = 0;
      lcd.setCursor(0, 0);        // １行目
      lcd.print("ClrStart ");    
      delay(1000);                // 時間待ち
      mes_cnt = 0;                // 測定数クリア
      eepwr2(EP_MESCNT, mes_cnt);
      chop_cnt = 0L;              // リレー開閉回数クリア
      eepwr4(EP_CHOPCNT, chop_cnt);
      dispmesdata();              // データ表示
      exc_j = J_RUN;              // 測定開始
      return;
    }
    if(f_swon){                   // 測定再開
      f_swon = 0;
      if(mes_cnt >= MESD_MAX){         // すでに最大データ数?
        lcd.setCursor(0, 0);
        lcd.print("Mem max  ");    
        return;                        // スタートしない
      }
      lcd.setCursor(0, 0);
      lcd.print("Run      ");    
      exc_j = J_RUN;              // 測定実行開始
      return;
    }
//  モニター表示
    if(tm_1ms == 0){              // 0.2秒ごと
      tm_1ms = 200;
      f_adon = 1;                 // A/D平均開始
    }
    if(f_adok){                   // A/Dデータ画定？
      f_adok = 0;
      dispmesdata();              // データ表示
    }
}

/*****  データ送信          *****/
void jdtx(void)
{
word n, d;
char s[16];
    lcd.setCursor(0, 0);          // １行目
    lcd.print("Data TX  ");    
    if(mes_cnt == 0){             // データ有無チェック
      lcd.setCursor(9, 0);        // １行目
      lcd.print("No Data");    
    }
//  データ出力loop
    else{
      Serial.println("Cyc mOhm");         // タイトル
      for(n = 0; n < mes_cnt; n++){
        d = eeprd2(EP_MESDTOP + (2 * n));  // 保存データ
        sprintf(s, "%3u/", n+1);           // データ位置+1
        lcd.setCursor(9, 0);
        lcd.print(s);    
        sprintf(s, "%4u", d);              // 測定データ mΩ値
        lcd.setCursor(0, 1);
        lcd.print(s);
        sprintf(s, "%3u %4u", n, d);       // シリアル出力
        Serial.println(s);
        delay(50);              // 時間待ち
      }
    }
    delay(1000);
    exc_j = J_STBY;             // スタンバイへ  
}

/*****  リレー開閉実行      *****/
//      第1回目の測定確認
void jrun(void)
{
    f_adon = 0;                 // A/D変換中止
    f_adok = 0;
    dispmesdata();              // データ表示
    if(mes_cnt == 0){           // はじめての測定?
      exc_j = J_MES;            // 接触抵抗測定へ  
    }
    else{
      exc_j++;                  // 電源接続へ
    }
}

/*****  リレー開閉実行＃１     *****/
//      AC電源接続
void jrun1(void)
{
    f_pwrlon = 1;                 // 電源接続オン
    lcd.setCursor(0, 0);
    lcd.print("Run      ");    
    exc_j++;
}  
  
/*****  リレー開閉実行＃２     *****/
//      接点開閉 on
void jrun2(void)
{
    if(f_pwrlon == 0){               // 電源オン完了を待つ
      digitalWrite(RL_A, HIGH);      // 接点試験リレーon
      tm_1ms = random(TM_RLCHOP, TM_RLCHOP+3);  // 乱数を使う
      exc_j++;
    }
}  

/*****  リレー開閉実行＃３     *****/
//      接点開閉 on 安定待ち
void jrun3(void)
{
    if(tm_1ms == 0){
      digitalWrite(RL_A, LOW);     // 接点試験リレーoff
      tm_1ms = random(TM_RLCHOP, TM_RLCHOP+3);  // 乱数を使う
      exc_j++;
    }
}

/*****  リレー開閉実行＃４     *****/
//      接点開閉 off 安定待ち
void jrun4(void)
{
    if(f_swon){                   // 一時停止?
      f_swon = 0;
      f_swstart = 0;              // 他のSW入力もキャンセル
      f_swtx    = 0;
      eepwr4(EP_CHOPCNT, chop_cnt);   // 開閉回数記録
      exc_j = J_STBY;                 // スタンバイへ 
      return;
    }
    if(tm_1ms == 0){              // off タイムアップ
      chop_cnt++;                 // 開閉回数+1
      dispmesdata();              // データ表示
      if((chop_cnt % RL_DCYC) == 0){     // 表示サイクル
        exc_j = J_MES;                   // 接触抵抗測定へ  
      }
      else{
        exc_j = J_RUN2;                  // on繰り返し
      }
    }
}

/*****  測定      *****/
//      AC電源接続をオフ
void jmes(void)
{
    f_pwrloff = 1;                // 電源接続オフ
    mrtry_cnt = 0;                // 測定安定待ちリトライ回数クリア
    lcd.setCursor(0, 0);          // １行目
    lcd.print("Measure  ");    
    exc_j++;
}

/*****  測定＃１      *****/
//      測定リレーをオン
void jmes1(void)
{
    if(f_pwrloff == 0){             // オフ待ち
      f_mesrl = 1;                  // 接点試験リレー処理開始
      exc_j++;
    }
}

/*****  測定＃２      *****/
//      A/D完了待ち      
void jmes2(void)
{
char s[20];
    if(f_mesrl == 0){              // A/D値確定
      ad_mem = ad_data;            // 記録チェックデータ
      dispmesdata();               // データ表示
//  シリアル出力
      sprintf(s, "%8lu %4u", chop_cnt, ad_data);
      Serial.println(s);
//  初回の測定はすぐEEPROMに書き込み
      if(mes_cnt == 0){           // はじめての測定?
        exc_j = J_MWR;            // EEPROM書き込みへ
        return;  
      }
//  データセーブ？
      if((chop_cnt % RL_MCYC) == 0){    // 測定記録サイクル
        tm_1sec = TM_MWAIT;         // 30秒待ち
        exc_j = J_MEM;              // データ記録処理へ
      }
      else{                         // 記録しなければ
        exc_j = J_RUN1;             // 電源接続してon繰り返し
      }
    }
}

/*****  測定値記録        *****/
//      接点の状態安定待ち
void jmem(void)
{
char s[16];
    if(tm_1ms == 0){              // 0.2秒ごと
      tm_1ms = 200;
      sprintf(s, "Wait %-4u", tm_1sec);  // 待ち時間
      lcd.setCursor(0, 0);
      lcd.print(s);    
    }
//  時間経過チェック
    if(tm_1sec == 0){               // タイムアップ
      f_mesrl = 1;                  // 接点試験リレー処理開始
      exc_j++;
    }
}

/*****  測定値記録＃１        *****/
//      A/D変換待ち
void jmem1(void)
{
char s[24];
    if(f_mesrl == 0){              // A/D値確定
      dispmesdata();               // データ表示
      if(mrtry_cnt < 1000){        // 測定安定待ちリトライ回数+1
        mrtry_cnt++;
      }
//  シリアル出力　リトライ回数を付加
      sprintf(s, "%8lu %4u %4u", chop_cnt, ad_data, mrtry_cnt);
      Serial.println(s);
//    A/D安定チェック　数値が低下したらリトライ
      if(ad_data >= ad_mem){       // 低下が止まった
        exc_j = J_MWR;             // データ記録処理へ
      }
      else{                        // 抵抗値が低下したらリトライ
        ad_mem = ad_data;          // 新A/D値
        tm_1sec = TM_MRTRY;        // リトライ時間
        exc_j = J_MEM;             // データ記録リトライ
      }
    }
}

/*****  測定値EEPROM書き込み      *****/
void jmwr(void)
{
    eepwr2(EP_MESDTOP + (2 * mes_cnt), ad_data);  // A/D値保存
    mes_cnt++;                                    // データ数+1
    dispmesdata();                                // データ表示
    eepwr2(EP_MESCNT, mes_cnt);                   // データ数保存
    eepwr4(EP_CHOPCNT, chop_cnt);                 // 開閉回数
    if(mes_cnt >= MESD_MAX)          // 記録数maxでおわり?
          exc_j = J_STBY;            // スタンバイに
    else                             // まだメモリに余裕がある
          exc_j = J_RUN1;            // 電源接続してon繰り返し
}


/*****  実行テーブル    *****/
void (*tblexc[])(void)={
        jttl,                //    0:ﾀｲﾄﾙ表示
        jttl1,               //    1:ﾀｲﾄﾙ表示完了待ち
        jstby,               // *  2:スタンバイ
        jstby1,              //    3:スタンバイ#1
        jswchk,              // *  4:スイッチチェック
        jdtx,                // *  5:データ送信
        jrun,                // *  6:リレー接点開閉実行
        jrun1,               // *  7:リレー接点開閉実行#1
        jrun2,               // *  8:リレー接点開閉実行#2
        jrun3,               //    9:リレー接点開閉実行#3
        jrun4,               //   10:リレー接点開閉実行#4
        jmes,                // * 11:接触抵抗測定
        jmes1,               //   12:接触抵抗測定#1
        jmes2,               //   13:接触抵抗測定#2
        jmem,                // * 14:接触抵抗記録
        jmem1,               //   15:接触抵抗記録
        jmwr,                // * 16:接触抵抗記録
};



/******************************/
/*      セットアップ         */
/******************************/
void setup()
{
    lcd.createChar(CG_OHM, cgd_ohm);    // Ω文字
    lcd.begin(16, 2);                   // 液晶
    lcd.noCursor();
    pinMode(RL_A, OUTPUT);              // リレー駆動
    pinMode(RL_B, OUTPUT);
    pinMode(RL_C, OUTPUT);
    pinMode(AD_CLK,  OUTPUT);           // A/Dコンバータ
    pinMode(AD_DOUT, INPUT_PULLUP);
    pinMode(AD_DIN,  OUTPUT);
    pinMode(AD_CS,   OUTPUT);
    pinMode(SW_START, INPUT_PULLUP);    // SW入力 プルアップ有
    pinMode(SW_TX,    INPUT_PULLUP);
    digitalWrite(AD_CS,  HIGH);         // A/D CS  = H
    digitalWrite(AD_CLK, HIGH);         // A/D CLK = H
    digitalWrite(AD_DIN, HIGH);         // A/D DIN = H
//  EEPRON読み出し
    mes_cnt  = eeprd2(EP_MESCNT);       // 測定データ数
    chop_cnt = eeprd4(EP_CHOPCNT);      // リレー開閉回数
    if(mes_cnt > MESD_MAX){             // 最大数越えれば
      mes_cnt = 0;                      // 二つを初期化
      eepwr2(EP_MESCNT, mes_cnt);
      chop_cnt = 0L;
      eepwr4(EP_CHOPCNT, chop_cnt);
    }
//   シリアル通信
    Serial.begin(9600);        
//  タイマー割り込み
    MsTimer2::set(1, int1ms);           // 1ms割り込み
    MsTimer2::start();                  // 割り込み処理開始
}


/******************************/
/*      ループ                */
/******************************/
void loop()
{
    adavr();                    // A/D変換処理
    swchkstart();               // スタートスイッチチェック
    swchktx();                  // データ送信スイッチチェック
    pwrlon();                   // 電源接続リレーオン処理
    pwrloff();                  // 電源接続リレーオフ処理
    mesrl();                    // 接点試験リレー+A/D変換
    tblexc[exc_j]();            // テーブル実行
}


//====  end of "RLcontact1"  =====

